package org.springmodules.javaspaces;
/*
 * Copyright 2002-2006 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.config.AbstractFactoryBean;
import org.springframework.core.task.TaskExecutor;

/**
 * Simple factoryBean used for starting delegate workers at startup. Requires a
 * TaskExecutor for executing the actual Runnable.
 * 
 * @author Costin Leau
 * 
 */
public abstract class AbstractDelegatingWorkerFactoryBean extends AbstractFactoryBean implements InitializingBean {

	private TaskExecutor taskExecutor;
	private Runnable[] tasks;
	private int concurrentThreadsPerTask = 1;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.springframework.beans.factory.config.AbstractFactoryBean#createInstance()
	 */
	protected Object createInstance() throws Exception {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.springframework.beans.factory.FactoryBean#getObjectType()
	 */
	public Class getObjectType() {
		return null;
	}

	public void afterPropertiesSet() throws Exception {
		if (taskExecutor == null)
			throw new IllegalArgumentException("taskExecutor is required");
		tasks = retrieveTasks();
		startTaskExecution(tasks);
	}

	protected void startTaskExecution(Runnable[] givenTasks) {
		// for each task
		for (int i = 0; i < givenTasks.length; i++) {
			// fire the number of conccurent threads
			for (int j = 0; j < concurrentThreadsPerTask; j++) {
				taskExecutor.execute(givenTasks[i]);
			}
		}
	}

	protected abstract Runnable[] retrieveTasks();

}
